from jinja2 import Template,Environment
def render_html(link,**kwargs):
    global env
    link1="templates\\"+link
    f = open(link1, "r", encoding="UTF8")
    text=f.read()
    try:
        return env.from_string(text).render(**kwargs)
    except:
        return Template(text).render(**kwargs)
def addfilter(name):
    def addf(fuc):
        global env
        try:
            env
        except:
            env=Environment()
        env.filters[name] = fuc
    return addf


from wsgiref.simple_server import make_server
import os
import re
import json
import datetime
import threading
html3=''
def cookie():
    while True:
        with open(".\\cookies.txt","r",encoding="UTF-8") as f2:
            newlist2=[]
            for i in f2.readlines():
                nowtime=datetime.datetime.now()
                every_cookie=eval(i.split("\n")[0])
                if str(every_cookie["time_to_live"]-nowtime)[0]=="-":
                    pass
                else:
                    newlist2.append(i)
            try:
                check2=tuple(newlist2[-1].split("\n"))
                if "\n" in check2:
                    newlist2[-1]=check2[0]
            except:
                pass
        with open(".\\cookies.txt","w", encoding="UTF8") as f2:
            f2.writelines(newlist2)
def makedict(env,re=None):
    HTTPdict={}
    HTTPdict["URLmatch"]=re
    HTTPdict["method"]=env['REQUEST_METHOD']
    HTTPdict["req_ip"]=env['REMOTE_ADDR']
    req_json=None
    try:
        req_body = env['wsgi.input'].read(int(env['CONTENT_LENGTH']))
        req_json = json.loads(req_body)
    except:
        pass
    HTTPdict["req_json"]=req_json
    return HTTPdict
def load_staic(link="."):
    link+="\\staic"        
    list1=[]
    for staicname in os.listdir(link):
        global staic_header
        staic_header="application/octet-stream"
        with open(link+"\\"+staicname, "r", encoding="UTF8") as f:
            houzhui=staicname.split(".")[-1]
            if houzhui=="js":
                staic_header="application/x-javascript"
            elif houzhui=="css":
                staic_header="text/css"
            try:
                list1.append(("/"+staicname,f.read(),staic_header,"text"))
            except UnicodeDecodeError:
                f.close()
                with open(link+"\\"+staicname,"rb") as f:
                    houzhui=staicname.split(".")[-1]
                    if houzhui=="ico":
                        staic_header="application/x-ico"
                    elif houzhui=="icon":
                        staic_header="application/x-icon"
                    list1.append(("/"+staicname,f.read(),staic_header,"byte"))
            
    return list1
def errorhandle(status,status2,start_response,env):
    global err
    global url
    try:
        start_response(status2, [('Content-Type', 'text/html;charset=utf-8')])
    except:
        pass
    if err==0:
        return "<h1>"+status2+"</h1>"
    HTTP_req_dict=makedict(env)
    return str(err(status))
def app(environ,start_response):
    global html3
    global url
    global staic
    try:
        global staic
        for url1 in url:
            if url1[0]==environ['PATH_INFO'] or (url1[4]=="RE" and not re.match(url1[0],environ['PATH_INFO'])==None):
                if environ['REQUEST_METHOD'] in url1[2]:
                    start_response('200 OK', url1[3])
                    match=None
                    if url1[0]==environ['PATH_INFO']:
                        pass
                    else:
                        match=re.findall(url1[0],environ['PATH_INFO'])
                    HTTP_req_dict= makedict(environ,match)
                    return [str(url1[1](HTTP_req_dict)).encode()]
                else:
                    return [str(errorhandle("405","405 Method Not Allowed",start_response,environ)).encode()]
        for staicdata in staic:
            if staicdata[0]==environ['PATH_INFO']:
                start_response('200 OK', [("Content-Type",staicdata[2])])
                if staicdata[3]=="byte":
                    return [staicdata[1]]
                return [staicdata[1].encode()]
        return [errorhandle("404","404 Not Found",start_response,environ).encode()]
    except Exception as e:
        print("==========================")
        print("Found a bug!")
        with open(".\\debug.txt", "a", encoding="UTF8") as f:
            if os.path.getsize(".\\debug.txt"):
                f.write("\n")
            f.write("==========================")
            f.write("\n")
            f.write(str(e))
            f.write("\n")
            f.write("==========================")
        print("see .\\debug.txt for more information")
        print("==========================")
        return [errorhandle("500","500 Internal Server Error",start_response,environ).encode()]

class _Dakada:
    def __init__(self,link=".",host='',port=8000):
        self.link=link
        self.host=host
        self.port=port
        self.url1=[]
        self.err1=0
        return 
    def addpath(self,path,URL_type="URL",method=['GET'],req_header=[('Content-Type', 'text/html;charset=utf-8')]):
        def returnfuc(fuc):
            self.url1.append((path,fuc,method,req_header,URL_type))
        return returnfuc
    def run(self):
        try:
            global url
            global err
            global p_link
            global staic
            err=self.err1
            url=self.url1
            p_link=self.link
            print("making server...")
            staic=load_staic(p_link)
            http=make_server(str(self.host),self.port,app)
            print("OK")
            print("starting cookiehandler...")
            t=threading.Thread(target=cookie)
            t.setDaemon(True)
            t.start()
            print("OK")
            print("Dakada(ver 1.8.4) is running!")
            if self.host=='':
                print("View at:http://localhost:"+str(self.port))
            else:
                print("View at:http://"+self.host+":"+str(self.port))
            http.serve_forever()
        except KeyboardInterrupt:
            print("server exit")
    def seterr(self,fuc):
        self.err1=fuc
    class db:
        def __init__(self,link=".",cookie=False):
            self.link=link
            self.dangerstr=["\n"]
            if not cookie:
                self.link+="\\db"
        def adddangerstr(self,dangerlist):
            self.dangerstr+=dangerlist
        def add(self,dblink,jsonordict_list):
            with open(self.link+"\\"+dblink, "a", encoding="UTF8") as f:
                i=1
                if os.path.getsize(self.link+"\\"+dblink):
                    f.write("\n")
                for every_json in jsonordict_list:
                    newlist=[danger for danger in dangerstr if danger in str(every_json)]
                    if not newlist==[]:
                        return "Bad data!"
                    f.write(str(every_json))
                    if not i==len(jsonordict_list):
                        f.write("\n")
                    i+=1
        def load(self,dblink,key):
            returnlist=[]
            with open(self.link+"\\"+dblink,"r", encoding="UTF8") as f:
                list1=f.readlines()
                for every_json in list(list1):
                    every_json2=eval(every_json.split("\n")[0])
                    dictkey=list(every_json2.keys())[0]
                    dictvalue=list(every_json2.values())[0]
                    if key==dictkey:
                        returnlist.append(every_json2)
            return returnlist
        def save(self,dblink,key,value):
            with open(self.link+"\\"+dblink,"r", encoding="UTF8") as f:
                list1=f.readlines()
                i=0
                for every_json in list(list1):
                    every_json2=eval(every_json.split("\n")[0])
                    dictkey=list(every_json2.keys())[0]
                    if dictkey==key:
                        list1[i]=str({dictkey:value})+"\n"
                    i+=1
            with open(self.link+"\\"+dblink,"w", encoding="UTF8") as f:
                f.writelines(list1)
        def delete(self,dblink,key):
            with open(self.link+"\\"+dblink,"r", encoding="UTF8") as f:
                list1=f.readlines()
                i=0
                newlist=[]
                length=len(list1)
                for every_json in range(length):
                    every_json2=eval(list1[every_json].split("\n")[0])
                    dictkey=list(every_json2.keys())[0]
                    if dictkey==key:
                        pass
                    else:
                        newlist.append(list1[i])
                    i+=1
                try:
                    check=tuple(newlist[-1].split("\n"))
                    if "\n" in check:
                        newlist[-1]=check[0]
                except:
                    pass
            with open(self.link+"\\"+dblink,"w", encoding="UTF8") as f:
                f.writelines(newlist)
        def clear(self,dblink):
            with open(self.link+"\\"+dblink,"w", encoding="UTF8") as f:
                f.writelines([])
import sys
sys.modules[__name__]=_Dakada


import os
from jinja2 import Template,Environment
env=Environment()
def getkey(value1):
    return ",".join(list(value1.keys()))
def getvalue(value1):
    return ",".join(list(value1.values()))
env.filters['getkey'] = getkey
env.filters['getvalue'] = getvalue
def make_admin(HTML=True,link="."):
    link+="\\db"
    list1=[]
    for json in os.listdir(link):
        with open(link+"\\"+json, "r", encoding="UTF8") as f:
            list2=f.readlines()
            list3=[]
            for json2 in list2:
                list3.append(eval(json2.split("\n")[0]))
        list1.append({"title":json.split(".")[0],"text":list3})
    if HTML:
        admin_template='''
        <!DOCTYPE html>
    {%for json in list1%}
        <h1>{{json.title}}</h1><br/>
        <style>
table{
    width: 40%;
    border-collapse: collapse;
}

table caption{
    font-size: 2em;
    font-weight: bold;
    margin: 1em 0;
}

th,td{
    border: 1px solid #999;
    text-align: center;
    padding: 20px 0;
}

table thead tr{
    background-color: #000000;
    color: #fff;
}

table tbody tr:nth-child(odd){
    background-color: #eee;
}





table tfoot tr td{
    text-align: right;
    padding-right: 20px;
}
    </style>
        <table border="1">
        <thead>
            <tr>
                <th>key</th>
                <th>value</th>
            </tr>
        </thead>
        <tbody>
            {%for every_json in json.text%}
                <tr>
                    <td>{{every_json|getkey}}</td>
                    <td>{{every_json|getvalue}}</td>
                </tr>
            {%endfor%}
        </table>
    {%endfor%}
                    '''
        return env.from_string(admin_template).render(list1=list1)
    else:
        return list1


from dakada.Dakada import db
from dakada import cookie
def getobj(p_link):
    global database
    global c_h
    try:
        database
        c_h
    except:
        database=db(p_link)
        c_h=cookie(p_link)
    finally:
        return (database,c_h)
def create_user_table(p_link):
    with open(p_link+"\\db\\User.json","a+",encoding="UTF-8") as f:
        pass
def register(username,passwd,p_link=".",**another):
    create_user_table(p_link)
    database,c_h=getobj(p_link)
    if database.add("User.json",[{"user":username,"passwd":passwd,"any":another}])=="Bad data!":
        return "failed"
def login(ip,username,passwd,time,p_link=".",**another):
    create_user_table(p_link)
    database,c_h=getobj(p_link)
    mylist=database.load("User.json","user")
    if mylist==[]:
        return False
    else:
        for my in mylist:
            if str(my)==str(dict(user=username,passwd=passwd,any=another)):
                c_h.add_cookies(ip,"user",username,time)
                return True
            else:
                continue
        return False
def logout(ip,p_link="."):
    database,c_h=getobj(p_link)
    c_h.delete_cookies(ip)



from dakada.Dakada import db
import datetime
class cookie:
    def __init__(self,p_link="."):
        self.link=p_link+"\\"
        self.cookiehandler=db(self.link,True)
    def add_cookies(self,ip,key,value,TTL):
        self.cookiehandler.add("cookies.txt",[{ip:{key:value},"time_to_live":datetime.datetime.now()+TTL}])
    def load_cookies(self,ip):
        return self.cookiehandler.load("cookies.txt",ip)
    def delete_cookies(self,ip):
        self.cookiehandler.delete("cookies.txt",ip)
import sys
sys.modules[__name__]=cookie